<?php
/**
*   Handler class for Mail Attachments
*
*   @author Viktor Zeman
*   @copyright Copyright (c) Quality Unit s.r.o.
*   @package SupportCenter
*/

QUnit::includeClass("QUnit_Rpc_Service");
class App_Service_Attachments extends QUnit_Rpc_Service {

    function initMethod($params) {
        $method = $params->get('method');

        switch($method) {
        	case 'getKBAttachmentsList':
        		return $this->callService('Users', 'authenticateAgent', $params); 
             default:
                return false;
                break;
        }
    }
    
    function insertMailAttachment($params) {
    	$db = $this->state->get('db'); 
    	$response =& $this->getByRef('response');
    	
    	if (!$params->checkFields(array('mail_id', 'file_id'))) {
    		$response->set('error', $this->state->lang->get('noIdProvided'));
    		return false;
    	}
    	
    	$params->set('table', 'mail_attachments');
    	return $this->callService('SqlTable', 'insert', $params);
    }
    
    function getMailAttachmentsList($params) {
        $db =& $this->state->getByRef('db');

        $params->set('columns', "*");
        $params->set('from', "mail_attachments");
        $where = "1";
    	if($id = $params->get('file_id')) {
    		$where .= " and file_id = '".$db->escapeString($id)."'";
    	}
    	if($id = $params->get('mail_id')) {
    		$where .= " and mail_id = '".$db->escapeString($id)."'";
    	}
    	$params->set('where', $where);
        $params->set('table', 'mail_attachments');
        return $this->callService('SqlTable', 'select', $params);
    }
    
    function checkMailAttachmentRights($file_id) {
    	$session = QUnit::newObj('QUnit_Session');
    	$db = $this->state->get('db'); 
    	$response =& $this->getByRef('response');
    	
    	
    	//check if it is not file from KB
   		$sql = 'SELECT count(*) as access_test from kb_item_files kbf
				INNER JOIN kb_items kbi ON (kbf.item_id = kbi.item_id)
				WHERE kbf.file_id = \'' . $db->escapeString($file_id) . '\'';
    	$sth = $db->execute($sql);
        $this->state->log('debug', '(' . $sth->execution_time . ') ' . $sql, 'sql');
        if(QUnit_Object::isError($sth)) {
        	$this->state->log('error', $this->state->lang->get('dbError').$sth->get('errorMessage'), 'Attachment');
            return false;
        }
        $rs = QUnit::newObj('QUnit_Rpc_ResultSet');
        $rs->setRows($sth->fetchAllRows());
        $md = QUnit::newObj('QUnit_Rpc_MetaData', 'mail_attachments');
        $md->setColumnNames($sth->getNames());
        $md->setColumnTypes($sth->getTypes());
        $response->setResultVar('md', $md);
        $response->setResultVar('rs', $rs);
        $rows = $rs->getRows($md);

        if (count($rows) > 0 && $rows[0]['access_test'] > 0) {
			return true;
		}
    	
		$sql = '';
		//check if it is from ticket
    	if ($session->getVar('userType') == 'a') {
    		return true;
    	} else {
	    	if ($session->getVar('userType') == 'g') {
	    		$sql = 'SELECT count(*) as access_test FROM mail_attachments ma 
						INNER JOIN mails ON (mails.mail_id = ma.mail_id)
						INNER JOIN tickets ON (mails.ticket_id = tickets.ticket_id AND 
						(tickets.agent_owner_id = \'' . 
	    				$db->escapeString($session->getVar('userId')) . 
	    				'\' OR tickets.queue_id IN (
						SELECT queue_id FROM queues 
						WHERE public=\'y\' OR queues.queue_id IN (
						SELECT queue_id from queue_agents WHERE user_id=\'' . 
	    				$db->escapeString($session->getVar('userId')) . '\'))))
						WHERE ma.file_id = \'' . $db->escapeString($file_id) . '\'';
	    	} else if ($session->getVar('userType') == 'u') {
	    		$sql = 'SELECT count(*) as access_test from mail_attachments ma
						INNER JOIN mails ON (mails.mail_id = ma.mail_id)
						INNER JOIN tickets ON (mails.ticket_id = tickets.ticket_id)
						WHERE ma.file_id = \'' . $db->escapeString($file_id) . '\'
						AND tickets.customer_id = \'' . $db->escapeString($session->getVar('userId')) . '\'';
	    	}
			if (strlen($sql)) {	    	
		        $sth = $db->execute($sql);
		        $this->state->log('debug', '(' . $sth->execution_time . ') ' . $sql, 'sql');
		        if(QUnit_Object::isError($sth)) {
		        	$this->state->log('error', $this->state->lang->get('dbError').$sth->get('errorMessage'), 'Attachment');
		            return false;
		        }
		        $rs = QUnit::newObj('QUnit_Rpc_ResultSet');
		        $rs->setRows($sth->fetchAllRows());
		        $md = QUnit::newObj('QUnit_Rpc_MetaData', 'mail_attachments');
		        $md->setColumnNames($sth->getNames());
		        $md->setColumnTypes($sth->getTypes());
		        $response->setResultVar('md', $md);
		        $response->setResultVar('rs', $rs);
		        $rows = $rs->getRows($md);
		
		        if (count($rows) > 0 && $rows[0]['access_test'] > 0) {
					return true;
				}
			}
    	}
    	return false;
    }
    
    
    function insertKBAttachment($params) {
    	$db = $this->state->get('db'); 
    	$response =& $this->getByRef('response');
    	
    	if (!$params->checkFields(array('item_id', 'file_id'))) {
    		$response->set('error', $this->state->lang->get('noIdProvided'));
    		return false;
    	}
    	$params->set('table', 'kb_item_files');
    	return $this->callService('SqlTable', 'insert', $params);
    }
    
    function getKBAttachmentsList($params) {
        $db =& $this->state->getByRef('db');

        $params->set('columns', "*");
        $params->set('from', "kb_item_files kbf INNER JOIN files f ON (f.file_id = kbf.file_id)");
        $where = "1";
    	if($id = $params->get('file_id')) {
    		$where .= " and kbf.file_id = '".$db->escapeString($id)."'";
    	}
    	if($id = $params->get('item_id')) {
    		$where .= " and kbf.item_id = '".$db->escapeString($id)."'";
    	}
    	$params->set('where', $where);
        $params->set('table', 'kb_item_files');
        return $this->callService('SqlTable', 'select', $params);
    }
    
    
    function downloadFile($file_id, $download_type = 'attachment') {
    	if ($this->checkMailAttachmentRights($file_id)) {
    		$objFile = QUnit::newObj('App_Service_Files');
    		$objFile->state = $this->state;
    		$objFile->response = $this->response;
    		return $objFile->downloadFile($file_id, $download_type);
    	} else {
        	$this->state->log('error', $this->state->lang->get('noRightsForAttachmentDownload'), 'Attachment');
        	echo $this->state->lang->get('noRightsForAttachmentDownload');
        	return false;
    	}
    	return true;
    }
    
}
?>